import re
from dataclasses import dataclass
from typing import ClassVar, Dict, List

from sigma.rule import SigmaRule, SigmaLogSource, SigmaRuleBase

from sigma.validators.base import (
    SigmaRuleValidator,
    SigmaValidationIssue,
    SigmaValidationIssueSeverity,
)

from .config import ConfigHQ

config = ConfigHQ()


@dataclass
class SigmahqFilenameConventionIssue(SigmaValidationIssue):
    description: ClassVar[str] = "The rule filename doesn't match SigmaHQ convention"
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.MEDIUM
    filename: str


class SigmahqFilenameConventionValidator(SigmaRuleValidator):
    """Check a rule filename against SigmaHQ filename convention."""

    def validate(self, rule: SigmaRuleBase) -> List[SigmaValidationIssue]:
        filename_pattern = re.compile(r"[a-z0-9_]{10,90}\.yml")
        if rule.source is not None:
            filename = rule.source.path.name
            if filename_pattern.match(filename) is None or not "_" in filename:
                return [SigmahqFilenameConventionIssue([rule], filename)]
        return []


@dataclass
class SigmahqFilenamePrefixIssue(SigmaValidationIssue):
    description: ClassVar[str] = "The rule filename prefix doesn't match the SigmaHQ convention"
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.MEDIUM
    filename: str
    logsource: SigmaLogSource
    prefix: str


class SigmahqFilenamePrefixValidator(SigmaRuleValidator):
    """Check a rule filename against SigmaHQ filename prefix convention."""

    def validate(self, rule: SigmaRule) -> List[SigmaValidationIssue]:
        if rule.source is not None:
            filename = rule.source.path.name
            logsource = SigmaLogSource(
                category=rule.logsource.category,
                product=rule.logsource.product,
                service=rule.logsource.service,
            )

            if logsource in config.sigmahq_logsource_filepattern:
                if not filename.startswith(config.sigmahq_logsource_filepattern[logsource]):
                    return [
                        SigmahqFilenamePrefixIssue(
                            [rule],
                            filename,
                            rule.logsource,
                            config.sigmahq_logsource_filepattern[logsource],
                        )
                    ]
            else:
                # check only product but must exist
                if rule.logsource.product:
                    logsource = SigmaLogSource(
                        category=None, product=rule.logsource.product, service=None
                    )
                    if (
                        logsource in config.sigmahq_logsource_filepattern
                        and not filename.startswith(config.sigmahq_logsource_filepattern[logsource])
                    ):
                        return [
                            SigmahqFilenamePrefixIssue(
                                [rule],
                                filename,
                                rule.logsource,
                                config.sigmahq_logsource_filepattern[logsource],
                            )
                        ]
        return []



from dataclasses import dataclass
from typing import ClassVar, Dict, List

from sigma.rule import SigmaRule, SigmaLogSource, SigmaRuleBase
from sigma.validators.base import (
    SigmaRuleValidator,
    SigmaValidationIssue,
    SigmaValidationIssueSeverity,
)

from .config import ConfigHQ

config = ConfigHQ()


@dataclass
class SigmahqLogsourceUnknownIssue(SigmaValidationIssue):
    description: ClassVar[str] = "Rule uses an unknown logsource"
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.HIGH
    logsource: SigmaLogSource


class SigmahqLogsourceUnknownValidator(SigmaRuleValidator):
    """Checks if a rule uses an unknown logsource."""

    def validate(self, rule: SigmaRule) -> List[SigmaValidationIssue]:
        # Ensure rule is a SigmaRule instance to access logsource
        logsource = getattr(rule, "logsource", None)
        if logsource is not None:
            core_logsource = SigmaLogSource(
                category=getattr(logsource, "category", None),
                product=getattr(logsource, "product", None),
                service=getattr(logsource, "service", None),
            )
            if not core_logsource in config.sigma_fieldsname:
                return [SigmahqLogsourceUnknownIssue([rule], logsource)]
            else:
                return []
        else:
            return []


@dataclass
class SigmahqSysmonMissingEventidIssue(SigmaValidationIssue):
    description: ClassVar[str] = (
        "Rule uses the windows sysmon service logsource without the EventID field"
    )
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.HIGH


class SigmahqSysmonMissingEventidValidator(SigmaRuleValidator):
    """Checks if a rule uses the windows sysmon service logsource without the EventID field."""

    def validate(self, rule: SigmaRule) -> List[SigmaValidationIssue]:
        if rule.logsource.service == "sysmon":
            find = False
            for selection in rule.detection.detections.values():
                for item in selection.detection_items:
                    if item.field == "EventID":
                        find = True
            if find:
                return []
            else:
                return [SigmahqSysmonMissingEventidIssue([rule])]
        else:
            return []



from dataclasses import dataclass
import json
import os
import pathlib
from typing import Dict, List, Optional
from sigma.rule import SigmaLogSource
from .sigmahq_data import (
    ref_sigmahq_logsource_filepattern,
    ref_sigmahq_fieldsname,
    ref_sigmahq_redundant_field,
    ref_sigmahq_logsource_definition,
    ref_windows_provider_name,
    ref_windows_no_eventid,
)
import requests


def core_logsource(source: SigmaLogSource) -> SigmaLogSource:
    return SigmaLogSource(product=source.product, category=source.category, service=source.service)


def key_logsource(source: dict) -> str:
    product = source["product"] if source["product"] else "none"
    category = source["category"] if source["category"] else "none"
    service = source["service"] if source["service"] else "none"
    return f"{product}_{category}_{service}"


class ConfigHQ:
    """Loads SigmaHQ configuration from local JSON files if available, otherwise uses reference data."""

    JSON_FOLDER: str = "validator_json"
    JSON_NAME_TAXONOMY: str = "sigmahq_taxonomy.json"
    JSON_NAME_FILENAME: str = "sigmahq_filename.json"
    JSON_NAME_WINDOWS_PROVIDER: str = "sigmahq_windows_validator.json"

    taxonomy_version: str = "0.0.0"
    sigma_fieldsname: Dict[SigmaLogSource, List[str]] = {}
    sigmahq_redundant_fields: Dict[SigmaLogSource, List[str]] = {}

    filename_version: str = "0.0.0"
    sigmahq_logsource_filepattern: Dict[SigmaLogSource, str] = {}

    windows_version: str = "0.0.0"
    windows_no_eventid: List[str] = []
    windows_provider_name: Dict[SigmaLogSource, List[str]] = {}
    sigmahq_logsource_definition: Dict[SigmaLogSource, Optional[str]] = {}

    def __init__(self, config_dir: Optional[str] = None):
        # Accept both local path and remote URL for config_dir
        self.is_remote = False
        if config_dir and (config_dir.startswith("http://") or config_dir.startswith("https://")):
            self.config_dir = config_dir.rstrip("/")
            self.is_remote = True
        else:
            self.config_dir = (
                pathlib.Path(config_dir) if config_dir else pathlib.Path.cwd() / self.JSON_FOLDER
            )

        self._load_sigma_json()
        self._load_filename_json()
        self._load_windows_provider_json()

    def _load_json(self, filename: str):
        if self.is_remote:
            url = f"{self.config_dir}/{filename}"
            try:
                response = requests.get(url)
                response.raise_for_status()
                return response.json()
            except Exception as e:
                print(f"Error loading remote {filename}: {e}")
            return None
        else:
            config_dir_path = (
                pathlib.Path(self.config_dir)
                if isinstance(self.config_dir, str)
                else self.config_dir
            )
            path = config_dir_path / filename
            if path.exists():
                try:
                    with path.open("r", encoding="UTF-8") as file:
                        return json.load(file)
                except Exception as e:
                    print(f"Error loading {filename}: {e}")
            return None

    def _load_sigma_json(self):
        json_dict = self._load_json(self.JSON_NAME_TAXONOMY)
        if json_dict:
            taxonomy_info = dict()
            taxonomy_definition = dict()
            taxonomy_redundant = dict()
            temp = {key_logsource(v["logsource"]): v for v in json_dict["taxonomy"].values()}
            for key in sorted(temp.keys(), key=str.casefold):
                value = temp[key]
                logsource = core_logsource(SigmaLogSource.from_dict(value["logsource"]))
                fieldlist = []
                fieldlist.extend(value["field"]["native"])
                fieldlist.extend(value["field"]["custom"])
                taxonomy_info[logsource] = sorted(fieldlist, key=str.casefold)
                taxonomy_definition[logsource] = value["logsource"]["definition"]
                taxonomy_redundant[logsource] = value["field"]["redundant"]
            self.taxonomy_version = json_dict["version"]
            self.sigmahq_redundant_fields = taxonomy_redundant
            self.sigma_fieldsname = taxonomy_info
            self.sigmahq_logsource_definition = taxonomy_definition
        else:
            self.taxonomy_version = "0.0.0"
            self.sigmahq_redundant_fields = ref_sigmahq_redundant_field
            self.sigma_fieldsname = ref_sigmahq_fieldsname
            self.sigmahq_logsource_definition = ref_sigmahq_logsource_definition

    def _load_filename_json(self):
        json_dict = self._load_json(self.JSON_NAME_FILENAME)
        if json_dict:

            filename_info = dict()
            temp = {key_logsource(v["logsource"]): v for v in json_dict["pattern"].values()}
            for key in sorted(temp.keys(), key=str.casefold):
                value = temp[key]
                logsource = core_logsource(SigmaLogSource.from_dict(value["logsource"]))
                filename_info[logsource] = value["prefix"]
            self.filename_version = json_dict["version"]
            self.sigmahq_logsource_filepattern = filename_info
        else:
            self.filename_version = "0.0.0"
            self.sigmahq_logsource_filepattern = ref_sigmahq_logsource_filepattern

    def _load_windows_provider_json(self):
        json_dict = self._load_json(self.JSON_NAME_WINDOWS_PROVIDER)
        if json_dict:
            windows_provider_name = dict()
            for category in sorted(json_dict["category_provider_name"], key=str.casefold):
                windows_provider_name[
                    SigmaLogSource(product="windows", category=category, service=None)
                ] = json_dict["category_provider_name"][category]
            windows_no_eventid = sorted(json_dict["category_no_eventid"], key=str.casefold)
            self.windows_version = json_dict["version"]
            self.windows_provider_name = windows_provider_name
            self.windows_no_eventid = windows_no_eventid
        else:
            self.windows_version = "0.0.0"
            self.windows_provider_name = ref_windows_provider_name
            self.windows_no_eventid = ref_windows_no_eventid



from dataclasses import dataclass
import re
from typing import ClassVar, List
from sigma.conditions import ConditionIdentifier, ConditionItem, ConditionSelector
from sigma.correlations import SigmaCorrelationRule
from sigma.rule import SigmaDetections, SigmaRule, SigmaRuleBase
from sigma.validators.base import (
    SigmaValidationIssue,
    SigmaValidationIssueSeverity,
    SigmaRuleValidator,
)


@dataclass
class SigmahqOfthemConditionIssue(SigmaValidationIssue):
    description: ClassVar[str] = (
        "Rule uses the ' of them' keyword in the condition with only one selection in the detection section"
    )
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.LOW


class SigmahqOfthemConditionValidator(SigmaRuleValidator):
    """Check use of the ' of them' keyword with only a single selection in the detection section"""

    re_all_of_them: ClassVar[re.Pattern] = re.compile(r"\s+of\s+them")

    def validate(self, rule: SigmaRuleBase) -> List[SigmaValidationIssue]:
        if isinstance(rule, SigmaCorrelationRule):
            return []  # Correlation rules do not have detections

        detection = getattr(rule, "detection", None)
        if (
            detection is not None
            and hasattr(detection, "condition")
            and hasattr(detection, "detections")
            and any(self.re_all_of_them.search(condition) for condition in detection.condition)
            and len(detection.detections) == 1
        ):
            return [SigmahqOfthemConditionIssue([rule])]
        else:
            return []


@dataclass
class SigmahqOfselectionConditionIssue(SigmaValidationIssue):
    description: ClassVar[str] = (
        "Rule uses the 'All/X of ' format in the condition with only one selection in the detection section"
    )
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.LOW
    selection: str


class SigmahqOfselectionConditionValidator(SigmaRuleValidator):
    """Check use of the 'All/X of ' format with only one selection in the detection section"""

    re_x_of_them: ClassVar[re.Pattern] = re.compile(r"(?:\d+|all)\s+of\s+([^\s]+)")

    def validate(self, rule: SigmaRuleBase) -> List[SigmaValidationIssue]:
        if isinstance(rule, SigmaCorrelationRule):
            return []  # Correlation rules do not have detections

        detection = getattr(rule, "detection", None)
        if (
            detection is not None
            and hasattr(detection, "condition")
            and hasattr(detection, "detections")
        ):
            for condition in detection.condition:
                if self.re_x_of_them.search(condition):
                    all_name = self.re_x_of_them.findall(condition)
                    for name in all_name:
                        if name.startswith("filter_") and name.endswith("_*"):
                            continue

                        if name.startswith("selection_") and name.endswith("_*"):
                            continue

                        if name.endswith("_*"):
                            selection_count = 0
                            for selection_name in detection.detections:
                                if re.match(name, selection_name):
                                    selection_count += 1
                            if selection_count < 2:
                                return [SigmahqOfselectionConditionIssue([rule], name)]
        return []


@dataclass
class SigmahqMissingAsteriskConditionIssue(SigmaValidationIssue):
    description: ClassVar[str] = (
        "Rule uses a '1/all of ' keyword in the condition without an asterisk"
    )
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.MEDIUM
    selection: str


class SigmahqMissingAsteriskConditionValidator(SigmaRuleValidator):
    """Check the use of the '1/all of ' keyword without an asterisk in the condition"""

    re_x_of_them: ClassVar[re.Pattern] = re.compile(r"\s+of\s+([^\s\)]+)")

    def validate(self, rule: SigmaRuleBase) -> List[SigmaValidationIssue]:
        if isinstance(rule, SigmaCorrelationRule):
            return []  # Correlation rules do not have detections

        detection = getattr(rule, "detection", None)
        if detection is not None and hasattr(detection, "condition"):
            for condition in detection.condition:
                if self.re_x_of_them.search(condition):
                    all_name = self.re_x_of_them.findall(condition)
                    for name in all_name:
                        if name == "them":
                            continue
                        if not name.endswith("*"):
                            return [SigmahqMissingAsteriskConditionIssue([rule], name)]
        return []



# sigma/validators/sigmahq/tags.py

from dataclasses import dataclass
from typing import ClassVar, List, Tuple

from sigma.rule import SigmaRuleBase
from sigma.validators.base import (
    SigmaRuleValidator,
    SigmaValidationIssue,
    SigmaValidationIssueSeverity,
)
from .config import ConfigHQ

config = ConfigHQ()


@dataclass
class SigmahqTagsUniqueDetectionIssue(SigmaValidationIssue):
    description: ClassVar[str] = "Multiple 'detection' tags found."
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.HIGH


class SigmahqTagsUniqueDetectionValidator(SigmaRuleValidator):
    """Ensures that the tag.namespace 'detection' is unique in the tags."""

    def validate(self, rule: SigmaRuleBase) -> List[SigmaValidationIssue]:
        detection_tags = [tag for tag in rule.tags or [] if tag.namespace == "detection"]
        if len(detection_tags) > 1:
            return [SigmahqTagsUniqueDetectionIssue([rule])]
        return []


@dataclass
class SigmahqTagsDetectionIssue(SigmaValidationIssue):
    description: ClassVar[str] = "Rule doesn't have a detection tag."
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.LOW
    tag: str


@dataclass(frozen=True)
class SigmahqTagsDetectionValidator(SigmaRuleValidator):
    """Checks if a rule in a specific folder has the corresponding detection tag."""

    Folder_tag: Tuple[str, ...] = ("dfir", "emerging-threats", "threat-hunting")

    def validate(self, rule: SigmaRuleBase) -> List[SigmaValidationIssue]:
        if rule.source:
            for name in self.Folder_tag:
                if f"rules-{name}" in str(rule.source):
                    tag_found = False
                    for tag in rule.tags or []:
                        if tag.namespace == "detection" and tag.name == name:
                            tag_found = True
                            break
                    if not tag_found:
                        return [SigmahqTagsDetectionIssue([rule], tag=name)]
        return []


@dataclass
class SigmahqTagsUniqueTlpIssue(SigmaValidationIssue):
    description: ClassVar[str] = "Multiple 'tlp' tags found."
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.HIGH


class SigmahqTagsUniqueTlpValidator(SigmaRuleValidator):
    """Ensures that the tag.namespace 'tlp' is unique in the tags."""

    def validate(self, rule: SigmaRuleBase) -> List[SigmaValidationIssue]:
        tlp_tags = [tag for tag in rule.tags or [] if tag.namespace == "tlp"]
        if len(tlp_tags) > 1:
            return [SigmahqTagsUniqueTlpIssue([rule])]
        return []


@dataclass
class SigmahqTagsTlpIssue(SigmaValidationIssue):
    tlp: str
    description: ClassVar[str] = "The rule uses a non-authorized TLP."
    severity: ClassVar[SigmaValidationIssueSeverity] = SigmaValidationIssueSeverity.HIGH


@dataclass(frozen=True)
class SigmahqTagsTlpValidator(SigmaRuleValidator):
    """Checks if a rule uses a non-authorized TLP tag."""

    allowed_tlp: Tuple[str, ...] = ("clear",)

    def validate(self, rule: SigmaRuleBase) -> List[SigmaValidationIssue]:
        for tag in rule.tags or []:
            if tag.namespace == "tlp" and tag.name not in self.allowed_tlp:
                return [SigmahqTagsTlpIssue([rule], tlp=tag.name)]
        return []
